@@ -23,6 +23,7 @@
 
 #include "CaretLogger.h"
 #include "CaretOMP.h"
+#include "NiftiIO.h"
 #include "Vector3D.h"
 #include "WarpfieldFile.h"
 
@@ -46,7 +47,7 @@ OperationParameters* AlgorithmVolumeWarpfieldResample::getParameters()
     
     ret->addStringParameter(2, "warpfield", "the warpfield to apply");
     
-    ret->addVolumeParameter(3, "volume-space", "a volume file in the volume space you want for the output");
+    ret->addStringParameter(3, "volume-space", "a volume file in the volume space you want for the output");
     
     ret->addStringParameter(4, "method", "the resampling method");
     
@@ -68,7 +69,7 @@ void AlgorithmVolumeWarpfieldResample::useParameters(OperationParameters* myPara
 {
     VolumeFile* inVol = myParams->getVolume(1);
     AString warpName = myParams->getString(2);
-    VolumeFile* refSpace = myParams->getVolume(3);
+    AString refSpaceName = myParams->getString(3);
     AString method = myParams->getString(4);
     VolumeFile* outVol = myParams->getOutputVolume(5);
     OptionalParameter* fnirtOpt = myParams->getOptionalParameter(6);
@@ -90,9 +91,11 @@ void AlgorithmVolumeWarpfieldResample::useParameters(OperationParameters* myPara
     } else {
         throw AlgorithmException("unrecognized interpolation method");
     }
-    vector<int64_t> refDims;
-    refSpace->getDimensions(refDims);
-    AlgorithmVolumeWarpfieldResample(myProgObj, inVol, myWarpfield.getWarpfield(), refDims.data(), refSpace->getSform(), myMethod, outVol);
+    NiftiIO refSpaceIO;
+    refSpaceIO.openRead(refSpaceName);
+    vector<int64_t> refDims = refSpaceIO.getDimensions();
+    if (refDims.size() < 3) refDims.resize(3, 1);
+    AlgorithmVolumeWarpfieldResample(myProgObj, inVol, myWarpfield.getWarpfield(), refDims.data(), refSpaceIO.getHeader().getSForm(), myMethod, outVol);
 }
 
 AlgorithmVolumeWarpfieldResample::AlgorithmVolumeWarpfieldResample(ProgressObject* myProgObj, const VolumeFile* inVol, const VolumeFile* warpfield,
@@ -120,6 +123,10 @@ AlgorithmVolumeWarpfieldResample::AlgorithmVolumeWarpfieldResample(ProgressObjec
             *(outVol->getMapLabelTable(i)) = *(inVol->getMapLabelTable(i));
         }
     }
+    for (int64_t i = 0; i < numMaps; ++i)
+    {
+        outVol->setMapName(i, inVol->getMapName(i));
+    }
     for (int64_t c = 0; c < numComponents; ++c)
     {
         for (int64_t b = 0; b < numMaps; ++b)
