@@ -635,6 +635,7 @@ ModelSurfaceMontage::restoreModelSpecificInformationFromScene(const SceneAttribu
                  iter++) {
                 const int32_t key = iter->first;
                 const SceneClass* cerebellarSceneClass = dynamic_cast<const SceneClass*>(iter->second);
+                cerebellarSceneClass->setRestored(true); // prevents "failed to restore"
                 
                 CaretAssertArrayIndex(m_cerebellarConfiguration,
                                       BrainConstants::MAXIMUM_NUMBER_OF_BROWSER_TABS,
@@ -656,7 +657,7 @@ ModelSurfaceMontage::restoreModelSpecificInformationFromScene(const SceneAttribu
                  iter++) {
                 const int32_t key = iter->first;
                 const SceneClass* cerebralSceneClass = dynamic_cast<const SceneClass*>(iter->second);
-                
+                cerebralSceneClass->setRestored(true);// prevents "failed to restore"
                 CaretAssertArrayIndex(m_cerebralConfiguration,
                                       BrainConstants::MAXIMUM_NUMBER_OF_BROWSER_TABS,
                                       key);
@@ -677,6 +678,7 @@ ModelSurfaceMontage::restoreModelSpecificInformationFromScene(const SceneAttribu
                  iter++) {
                 const int32_t key = iter->first;
                 const SceneClass* flatClass = dynamic_cast<const SceneClass*>(iter->second);
+                flatClass->setRestored(true); // prevents "failed to restore"
                 
                 CaretAssertArrayIndex(m_flatMapsConfiguration,
                                       BrainConstants::MAXIMUM_NUMBER_OF_BROWSER_TABS,
@@ -924,6 +926,33 @@ ModelSurfaceMontage::restoreFromSceneVersionTwoAndEarlier(const SceneAttributes*
     }
 }
 
+/**
+ * Get the number of rows and columns in the displayed surface montage.
+ *
+ * @param tabIndex
+ *     Index of tab.
+ * @param numberOfRowsOut
+ *     Number of rows in the displayed montage.
+ * @param numberOfColumnsOut
+ *     Number of columns in the displayed montage.
+ */
+void
+ModelSurfaceMontage::getSurfaceMontageNumberOfRowsAndColumns(const int32_t tabIndex,
+                                                             int32_t& numberOfRowsOut,
+                                                             int32_t& numberOfColumnsOut) const
+{
+    numberOfRowsOut    = 1;
+    numberOfColumnsOut = 1;
+    
+    ModelSurfaceMontage* nonConstMSM = const_cast<ModelSurfaceMontage*>(this);
+    std::vector<const SurfaceMontageViewport*> surfaceMontageViewports;
+    nonConstMSM->getSurfaceMontageViewportsForTransformation(tabIndex,
+                                                             surfaceMontageViewports);
+    SurfaceMontageViewport::getNumberOfRowsAndColumns(surfaceMontageViewports,
+                                                      numberOfRowsOut,
+                                                      numberOfColumnsOut);
+}
+
 /**
  * Get the montage viewports for drawing by OpenGL.  The montage viewports
  * will be updated prior to returning them.   OpenGL will update
