@@ -21,6 +21,8 @@
  */
 /*LICENSE_END*/
 
+#include <array>
+
 #include "ApplicationTypeEnum.h"
 #include "BrainConstants.h"
 #include "CaretObject.h"
@@ -31,9 +33,11 @@ namespace caret {
     
     class Brain;
     class BrowserTabContent;
+    class BrowserWindowContent;
     class CaretPreferences;
     class CiftiConnectivityMatrixDataFileManager;
     class CiftiFiberTrajectoryManager;
+    class ImageCaptureSettings;
     class Model;
     
     /// Manages a Caret session which contains 'global' brain data.
@@ -52,7 +56,7 @@ namespace caret {
         
         int32_t getNumberOfBrains() const;
         
-        Brain* getBrain(const int32_t brainIndex);
+        Brain* getBrain(const int32_t brainIndex) const;
         
         CaretPreferences* getCaretPreferences();
         
@@ -64,6 +68,10 @@ namespace caret {
         
         const CiftiFiberTrajectoryManager* getCiftiFiberTrajectoryManager() const;
         
+        ImageCaptureSettings* getImageCaptureDialogSettings();
+        
+        const ImageCaptureSettings* getImageCaptureDialogSettings() const;
+        
         virtual SceneClass* saveToScene(const SceneAttributes* sceneAttributes,
                                         const AString& instanceName);
         
@@ -92,6 +100,9 @@ namespace caret {
         /** The browser tabs */
         BrowserTabContent* m_browserTabs[BrainConstants::MAXIMUM_NUMBER_OF_BROWSER_TABS];  
         
+        /** The browser window content */
+        std::array<BrowserWindowContent*, BrainConstants::MAXIMUM_NUMBER_OF_BROWSER_WINDOWS> m_browserWindowContent;
+        
         /** Holds valid models */
         std::vector<Model*> m_models;
         
@@ -107,6 +118,9 @@ namespace caret {
         /** Loads fiber trajectory data */
         CiftiFiberTrajectoryManager* m_ciftiFiberTrajectoryManager;
         
+        /** Settings for image capture dialog */
+        ImageCaptureSettings* m_imageCaptureDialogSettings;
+        
     };
     
 #ifdef __SESSION_MANAGER_DECLARE__
