@@ -389,6 +389,8 @@ SurfaceSelectionModel::saveToScene(const SceneAttributes* /*sceneAttributes*/,
     
     Surface* surface = getSurface();
     if (surface != NULL) {
+        sceneClass->addString("m_selectedSurfaceFullPath",
+                              surface->getFileName());
         sceneClass->addString("m_selectedSurface",
                               surface->getFileNameNoPath());
     }
@@ -416,21 +418,55 @@ SurfaceSelectionModel::restoreFromScene(const SceneAttributes* /*sceneAttributes
         return;
     }
     
+    std::vector<Surface*> allSurfaces = getAvailableSurfaces();
+    
+    const AString& surfaceFileNameFullPath = sceneClass->getStringValue("m_selectedSurfaceFullPath",
+                                                                        "");
+    /*
+     * For full path, find the best match using the right-most characters that 
+     * will contain any relative path.  When scene files are moved to different
+     * computers the full path may change the parts of the path nearest the 
+     * name of the file will match.
+     */
+    Surface* pathNameMatchSurface = NULL;
+    int32_t pathNameMatchLength = 0;
+    if ( ! surfaceFileNameFullPath.isEmpty()) {
+        for (auto surface : allSurfaces) {
+            const AString name = surface->getFileName();
+            const int32_t numMatch = name.countMatchingCharactersFromEnd(surfaceFileNameFullPath);
+            if (numMatch > pathNameMatchLength) {
+                pathNameMatchLength  = numMatch;
+                pathNameMatchSurface = surface;
+            }
+        }
+    }
+    
+    /*
+     * Match name of file with NO path
+     * Always restore this so that the object is marked as restored
+     * (within the 'get' method).  Otherwise if compiled debug, this
+     * object will get logged as 'not restored'.
+     */
+    Surface* nameMatchSurface = NULL;
     const AString& surfaceFileName = sceneClass->getStringValue("m_selectedSurface",
                                                                 "");
-    if (surfaceFileName.isEmpty() == false) {
-        std::vector<Surface*> surfaces = getAvailableSurfaces();
-        
-        for (std::vector<Surface*>::iterator iter = surfaces.begin();
-             iter != surfaces.end();
-             iter++) {
-            Surface* s = *iter;
-            if (s->getFileNameNoPath() == surfaceFileName) {
-                setSurface(s);
+    if ( ! surfaceFileName.isEmpty()) {
+        for (auto surface : allSurfaces) {
+            if (surface->getFileNameNoPath() == surfaceFileName) {
+                nameMatchSurface = surface;
                 break;
             }
         }
-        
+    }
+    
+    if (pathNameMatchSurface != NULL) {
+        setSurface(pathNameMatchSurface);
+    }
+    else if (nameMatchSurface != NULL) {
+        setSurface(nameMatchSurface);
+    }
+    else {
+        setSurface(NULL);
     }
 }
 
