@@ -20,6 +20,8 @@
 
 #include "CommandOperation.h"
 
+#include "CaretAssert.h"
+
 using namespace caret;
 
 /**
@@ -35,6 +37,11 @@ CommandOperation::CommandOperation(const AString& commandLineSwitch,
 {
     this->commandLineSwitch = commandLineSwitch;
     this->operationShortDescription = operationShortDescription;
+    CaretAssert(commandLineSwitch != "-command-switch");//catch failure to change things in the command template
+    for (int i = 0; i < commandLineSwitch.length(); ++i)//release build should optimize out empty loops
+    {
+        CaretAssert(commandLineSwitch[i].unicode() < 128);
+    }
 }
  
 /**
@@ -67,6 +74,19 @@ void CommandOperation::disableProvenance()
 {
 }
 
+void CommandOperation::setCiftiOutputDTypeAndScale(const int16_t&, const double&, const double&)
+{
+}
+
+void CommandOperation::setCiftiOutputDTypeNoScale(const int16_t&)
+{
+}
+
+AString CommandOperation::doCompletion(ProgramParameters&, const bool&)
+{
+    return "";
+}
+
 bool CommandOperation::takesParameters()
 {
     return true;
