@@ -61,7 +61,8 @@ DataFileTypeEnum::DataFileTypeEnum(const Enum enumValue,
                                    const bool fileIsUsedWithOneStructure,
                                    const AString& fileExtensionOne,
                                    const AString& fileExtensionTwo,
-                                   const AString& fileExtensionThree)
+                                   const AString& fileExtensionThree,
+                                   const AString& fileExtensionFour)
 {
     this->enumValue = enumValue;
     this->integerCode = DataFileTypeEnum::integerCodeGenerator++;
@@ -79,6 +80,9 @@ DataFileTypeEnum::DataFileTypeEnum(const Enum enumValue,
     if (fileExtensionThree.isEmpty() == false) {
         this->fileExtensions.push_back(fileExtensionThree);
     }
+    if (fileExtensionFour.isEmpty() == false) {
+        this->fileExtensions.push_back(fileExtensionFour);
+    }
 
     AString filterText = this->guiName + " Files (";
     
@@ -114,100 +118,123 @@ DataFileTypeEnum::initialize()
     }
     initializedFlag = true;
 
-    enumData.push_back(DataFileTypeEnum(BORDER, 
+    enumData.push_back(DataFileTypeEnum(ANNOTATION,
+                                        "ANNOTATION",
+                                        "Annotation",
+                                        "ANNOTATION",
+                                        false,
+                                        "annot",
+                                        "wb_annot"));
+    
+    enumData.push_back(DataFileTypeEnum(ANNOTATION_TEXT_SUBSTITUTION,
+                                        "ANNOTATION_TEXT_SUBSTITUTION",
+                                        "Annotation Text Substitution",
+                                        "ANNOTATION TEXT SUBSTITUTION",
+                                        false,
+                                        "wb_annsub.csv"));
+    
+    enumData.push_back(DataFileTypeEnum(BORDER,
                                         "BORDER", 
                                         "Border",
                                         "BORDER",
                                         true,
-                                        "border"));
+                                        "border",
+                                        "wb_border"));
     
     enumData.push_back(DataFileTypeEnum(CONNECTIVITY_DENSE, 
                                         "CONNECTIVITY_DENSE", 
-                                        "Connectivity - Dense",
+                                        "CIFTI - Dense",
                                         "CONNECTIVITY",
                                         false,
                                         "dconn.nii"));
     
+    enumData.push_back(DataFileTypeEnum(CONNECTIVITY_DENSE_DYNAMIC,
+                                        "CONNECTIVITY_DENSE_DYNAMIC",
+                                        "CIFTI - Dense Dynamic",
+                                        "CONNECTIVITY DYNAMIC",
+                                        false,
+                                        "dynconn.nii"));
+    
     enumData.push_back(DataFileTypeEnum(CONNECTIVITY_DENSE_LABEL,
                                         "CONNECTIVITY_DENSE_LABEL",
-                                        "Connectivity - Dense Label",
+                                        "CIFTI - Dense Label",
                                         "CIFTI LABELS",
                                         false,
                                         "dlabel.nii"));
     
     enumData.push_back(DataFileTypeEnum(CONNECTIVITY_DENSE_PARCEL,
                                         "CONNECTIVITY_DENSE_PARCEL",
-                                        "Connectivity - Dense Parcel",
+                                        "CIFTI - Dense Parcel",
                                         "CIFTI DENSE PARCEL",
                                         false,
                                         "dpconn.nii"));
     
     enumData.push_back(DataFileTypeEnum(CONNECTIVITY_DENSE_SCALAR,
                                         "CONNECTIVITY_DENSE_SCALAR",
-                                        "Connectivity - Dense Scalar",
+                                        "CIFTI - Dense Scalar",
                                         "CIFTI SCALARS",
                                         false,
                                         "dscalar.nii"));
     
     enumData.push_back(DataFileTypeEnum(CONNECTIVITY_DENSE_TIME_SERIES,
                                         "CONNECTIVITY_DENSE_TIME_SERIES", 
-                                        "Connectivity - Dense Data Series",
+                                        "CIFTI - Dense Data Series",
                                         "DATA SERIES",
                                         false,
                                         "dtseries.nii"));
     
     enumData.push_back(DataFileTypeEnum(CONNECTIVITY_FIBER_ORIENTATIONS_TEMPORARY,
                                         "CONNECTIVITY_FIBER_ORIENTATIONS_TEMPORARY",
-                                        "Connectivity - Fiber Orientations TEMPORARY",
+                                        "CIFTI - Fiber Orientations TEMPORARY",
                                         "FIBER ORIENTATION TEMPORARY",
                                         false,
                                         "fiberTEMP.nii"));
     
     enumData.push_back(DataFileTypeEnum(CONNECTIVITY_FIBER_TRAJECTORY_TEMPORARY,
                                         "CONNECTIVITY_FIBER_TRAJECTORY_TEMPORARY",
-                                        "Connectivity - Fiber Trajectory TEMPORARY",
+                                        "CIFTI - Fiber Trajectory TEMPORARY",
                                         "FIBER TRAJECTORY TEMPORARY",
                                         false,
                                         "trajTEMP.wbsparse"));
     
     enumData.push_back(DataFileTypeEnum(CONNECTIVITY_PARCEL,
                                         "CONNECTIVITY_PARCEL",
-                                        "Connectivity - Parcel",
+                                        "CIFTI - Parcel",
                                         "CIFTI PARCEL",
                                         false,
                                         "pconn.nii"));
     
     enumData.push_back(DataFileTypeEnum(CONNECTIVITY_PARCEL_DENSE,
                                         "CONNECTIVITY_PARCEL_DENSE",
-                                        "Connectivity - Parcel Dense",
+                                        "CIFTI - Parcel Dense",
                                         "CIFTI PARCEL DENSE",
                                         false,
                                         "pdconn.nii"));
     
     enumData.push_back(DataFileTypeEnum(CONNECTIVITY_PARCEL_LABEL,
                                         "CONNECTIVITY_PARCEL_LABEL",
-                                        "Connectivity - Parcel Label",
+                                        "CIFTI - Parcel Label",
                                         "CIFTI PARCEL LABEL",
                                         false,
                                         "plabel.nii"));
     
     enumData.push_back(DataFileTypeEnum(CONNECTIVITY_PARCEL_SCALAR,
                                         "CONNECTIVITY_PARCEL_SCALAR",
-                                        "Connectivity - Parcel Scalar",
+                                        "CIFTI - Parcel Scalar",
                                         "CIFTI PARCEL SCALAR",
                                         false,
                                         "pscalar.nii"));
     
     enumData.push_back(DataFileTypeEnum(CONNECTIVITY_PARCEL_SERIES,
                                         "CONNECTIVITY_PARCEL_SERIES",
-                                        "Connectivity - Parcel Series",
+                                        "CIFTI - Parcel Series",
                                         "CIFTI PARCEL SERIES",
                                         false,
                                         "ptseries.nii"));
     
     enumData.push_back(DataFileTypeEnum(CONNECTIVITY_SCALAR_DATA_SERIES,
                                         "CONNECTIVITY_SCALAR_DATA_SERIES",
-                                        "Connectivity - Scalar Data Series",
+                                        "CIFTI - Scalar Data Series",
                                         "CIFTI SCALAR DATA SERIES",
                                         false,
                                         "sdseries.nii"));
@@ -217,13 +244,16 @@ DataFileTypeEnum::initialize()
                                         "Foci",
                                         "FOCI",
                                         false,
-                                        "foci"));
+                                        "foci",
+                                        "wb_foci"));
     
     enumData.push_back(DataFileTypeEnum(IMAGE,
                                         "IMAGE",
                                         "Image",
                                         "IMAGE",
                                         false,
+                                        "jpg",
+                                        "jpeg",
                                         "png",
                                         "ppm"));
     
@@ -247,7 +277,8 @@ DataFileTypeEnum::initialize()
                                         "Palette",
                                         "PALETTE",
                                         false,
-                                        "palette"));
+                                        "palette",
+                                        "wb_palette"));
     
     enumData.push_back(DataFileTypeEnum(RGBA, 
                                         "RGBA", 
@@ -261,14 +292,16 @@ DataFileTypeEnum::initialize()
                                         "Scene",
                                         "SCENE",
                                         false,
-                                        "scene"));
+                                        "scene",
+                                        "wb_scene"));
     
     enumData.push_back(DataFileTypeEnum(SPECIFICATION, 
                                         "SPECIFICATION", 
                                         "Specification",
                                         "SPECIFICATION",
                                         false,
-                                        "spec"));
+                                        "spec",
+                                        "wb_spec"));
     
     enumData.push_back(DataFileTypeEnum(SURFACE, 
                                         "SURFACE", 
@@ -398,41 +431,34 @@ DataFileTypeEnum::toGuiName(Enum enumValue) {
 }
 
 /**
- * Get an enumerated value corresponding to its GUI name.
- * @param s 
- *     Name of enumerated value.
- * @param isValidOut 
- *     If not NULL, it is set indicating that a
- *     enum value exists for the input name.
- * @return 
+ * Get a short GUI string representation of the enumerated type.
+ * Removes any "Connectivity - " or " TEMPORARY" from the toGuiName().
+ * @param enumValue
  *     Enumerated value.
+ * @return
+ *     String representing enumerated value.
  */
-DataFileTypeEnum::Enum 
-DataFileTypeEnum::fromGuiName(const AString& guiName, bool* isValidOut)
+AString
+DataFileTypeEnum::toShortGuiName(Enum enumValue)
 {
-    if (initializedFlag == false) initialize();
+    const AString typeName = DataFileTypeEnum::toGuiName(enumValue);
     
-    bool validFlag = false;
-    Enum enumValue = UNKNOWN;
+    const AString connectivityPrefix("Connectivity - ");
+    const int connectivityPrefixLength = connectivityPrefix.length();
     
-    for (std::vector<DataFileTypeEnum>::iterator iter = enumData.begin();
-         iter != enumData.end();
-         iter++) {
-        const DataFileTypeEnum& d = *iter;
-        if (d.guiName == guiName) {
-            enumValue = d.enumValue;
-            validFlag = true;
-            break;
-        }
+    const AString temporarySuffix(" TEMPORARY");
+    const int temporarySuffixLength = temporarySuffix.length();
+    
+    AString text = typeName;
+    if (text.startsWith(connectivityPrefix)) {
+        text = text.mid(connectivityPrefixLength);
     }
     
-    if (isValidOut != 0) {
-        *isValidOut = validFlag;
+    if (text.endsWith(temporarySuffix)) {
+        text = text.left(text.length() - temporarySuffixLength);
     }
-    else if (validFlag == false) {
-        CaretAssertMessage(0, AString("guiName \"" + guiName + "\" failed to match enumerated value for type DataFileTypeEnum"));
-    }
-    return enumValue;
+    
+    return text;
 }
 
 /**
@@ -585,7 +611,8 @@ DataFileTypeEnum::getAllFileExtensions(const Enum enumValue)
 }
 
 /**
- * @return All valid file extensions for all file types except UNKNOWN.
+ * @return All valid file extensions for all file types except UNKNOWN
+ * and CONNECTIVITY_DENSE_DYNAMIC
  */
 std::vector<AString>
 DataFileTypeEnum::getFilesExtensionsForEveryFile()
@@ -595,7 +622,13 @@ DataFileTypeEnum::getFilesExtensionsForEveryFile()
     for (std::vector<DataFileTypeEnum>::iterator enumIter = enumData.begin();
          enumIter != enumData.end();
          enumIter++) {
-        if (enumIter->enumValue != DataFileTypeEnum::UNKNOWN) {
+        if (enumIter->enumValue == DataFileTypeEnum::CONNECTIVITY_DENSE_DYNAMIC) {
+            /* nothing */
+        }
+        else if (enumIter->enumValue == DataFileTypeEnum::UNKNOWN) {
+            /* nothing */
+        }
+        else {
             allExtensions.insert(allExtensions.end(),
                                  enumIter->fileExtensions.begin(),
                                  enumIter->fileExtensions.end());
@@ -797,22 +830,30 @@ DataFileTypeEnum::fromIntegerCode(const int32_t integerCode, bool* isValidOut)
  *
  * @param allEnums
  *     A vector that is OUTPUT containing all of the enumerated values.
- * @param includeUNKNOWN
- *     If true, the UNKNOWN enum is included.
+ * @param options
+ *     Bitwise mask for options
  */
 void
 DataFileTypeEnum::getAllEnums(std::vector<DataFileTypeEnum::Enum>& allEnums,
-                              const bool includeUnknown)
+                              const uint32_t options)
 {
     if (initializedFlag == false) initialize();
     
     allEnums.clear();
     
+    const bool includeDenseDynamicFlag = (options & OPTIONS_INCLUDE_CONNECTIVITY_DENSE_DYNAMIC);
+    const bool includeUnknownFlag      = (options & OPTIONS_INCLUDE_UNKNOWN);
+    
     for (std::vector<DataFileTypeEnum>::iterator iter = enumData.begin();
          iter != enumData.end();
          iter++) {
+        if (iter->enumValue == CONNECTIVITY_DENSE_DYNAMIC) {
+            if ( ! includeDenseDynamicFlag) {
+                continue;
+            }
+        }
         if (iter->enumValue == UNKNOWN) {
-            if ( ! includeUnknown) {
+            if ( ! includeUnknownFlag) {
                 continue;
             }
         }
