@@ -111,9 +111,14 @@ FileInformation::FileInformation(const AString& path,
          * Clean up path to remove any ".." (up a directory level).
          * Note that canonicalFilePath() will return an empty string
          * if the path does not point to a valid file.
+         * 
+         * TSC: NEVER USE CANONICAL FOR THIS!  canonical resolves symlinks, which is NOT DESIRED unless explicitly asked for (by calling a function including the word "canonical").
+         * 
          */
         if (getAbsoluteFilePath().contains("..")) {
-            const AString cleanedPath = m_fileInfo.canonicalFilePath();
+            AString cleanedPath = QDir::cleanPath(m_fileInfo.absolutePath());
+            if (!cleanedPath.endsWith('/')) cleanedPath += "/";//because "/" and "/usr" are both possible results
+            cleanedPath += m_fileInfo.fileName();
             if (cleanedPath.isEmpty() == false) {
                 m_fileInfo.setFile(cleanedPath);
             }
@@ -550,9 +555,9 @@ FileInformation::getFileComponents(AString& absolutePathOut,
  *     The file extension without the leading dot.
  */
 AString
-FileInformation::assembleFileComponents(AString& pathName,
-                                        AString& fileNameWithoutExtension,
-                                        AString& extensionWithoutDot)
+FileInformation::assembleFileComponents(const AString& pathName,
+                                        const AString& fileNameWithoutExtension,
+                                        const AString& extensionWithoutDot)
 {
     AString name;
     if ( ! pathName.isEmpty()) {
@@ -560,12 +565,95 @@ FileInformation::assembleFileComponents(AString& pathName,
     }
     name += fileNameWithoutExtension;
     if ( ! extensionWithoutDot.isEmpty()) {
-        name += ("." + extensionWithoutDot);
+        /*
+         * User might include dot at beginning of extension
+         */
+        if (extensionWithoutDot.startsWith(".")) {
+            name += extensionWithoutDot;
+        }
+        else {
+            name += ("." + extensionWithoutDot);
+        }
     }
 
     return name;
 }
 
+/**
+ * Assemble the file components into a file path and name.
+ *
+ * @param pathName
+ *     Path for file (may be absolute, relative, or empty).
+ * @param fileNameWithExtension
+ *     Name of file with extension.
+ */
+AString
+FileInformation::assembleFileComponents(const AString& pathName,
+                                        const AString& fileNameWithExtension)
+{
+    AString name;
+    if ( ! pathName.isEmpty()) {
+        name += (pathName + "/");
+    }
+    name += fileNameWithExtension;
+    
+    return name;
+}
+
+/**
+ * Create the file path for a file with the given name in the 
+ * computer system's temporary directory.
+ *
+ * @param fileName
+ *     Name of the file with an extension.  Any path in the name is ignored.
+ * @return
+ *     File path in the temporary directory with the given file name.
+ */
+AString
+FileInformation::createTemporaryFilePathName(const AString& fileName)
+{
+    AString nameOut;
+    
+    FileInformation fileInfo(fileName);
+    AString nameNoPath = fileInfo.getFileName();
+    if (nameNoPath.isEmpty()) {
+        nameOut = FileInformation::assembleFileComponents(SystemUtilities::getTempDirectory(), "name_missing.tmp");
+    }
+    else {
+        nameOut = FileInformation::assembleFileComponents(SystemUtilities::getTempDirectory(), nameNoPath);
+    }
+    
+    return nameOut;
+}
+
+/**
+ * Replace the extension in a file name.
+ *
+ * @param fileName
+ *     Name of file
+ * @param newExtension
+ *     New extension that replaces the extension in fileName
+ * @return
+ *     fileName with extension replaced with newExtension.
+ */
+AString
+FileInformation::replaceExtension(const AString& fileName,
+                                  const AString& newExtension)
+{
+    AString thePath, theName, theExtension;
+    
+    FileInformation fileInfo(fileName);
+    fileInfo.getFileComponents(thePath,
+                               theName,
+                               theExtension);
+    
+    const AString nameOut = assembleFileComponents(thePath,
+                                                   theName,
+                                                   newExtension);
+    return nameOut;
+}
+
+
 /**
  * Convert, if needed, the file information to a local, absolute path.
  *
@@ -731,4 +819,25 @@ FileInformation::fileSizeToStandardUnits(const int64_t numberOfBytes)
     return sizeString;
 }
 
+/**
+ * Cleans the path.  First, replaces any backward slashes with forward slashes
+ * and then calls QDir::cleanPath().   Note that QDir::cleanPath() only converts
+ * backward slashes to forward slashes when the native separator is a backward
+ * slash (on Windows).   Thus, if a path on Mac or Linux contains a backward
+ * slash, QDir::cleanPath() will not convert to a forward slash.
+ *
+ * @param path
+ *     This input path.
+ * @return
+ *     Path after cleaning.
+ */
+AString
+FileInformation::cleanPath(const AString& path)
+{
+    AString pathOut(path);
+    pathOut.replace(QLatin1Char('\\'), QLatin1Char('/'));
+    pathOut = QDir::cleanPath(pathOut);
+    return pathOut;
+}
+
 
