@@ -59,17 +59,24 @@ TileTabsConfiguration::~TileTabsConfiguration()
 
 /**
  * Copy constructor.
+ *
+ * NOTE: Unique identifier remains the same ! See also: newCopyWithNewUniqueIdentifier()
+ *
  * @param obj
  *    Object that is copied.
  */
 TileTabsConfiguration::TileTabsConfiguration(const TileTabsConfiguration& obj)
 : CaretObject(obj)
 {
+    initialize();
     this->copyHelperTileTabsConfiguration(obj);
 }
 
 /**
  * Assignment operator.
+ *
+ * NOTE: Unique identifier remains the same !  See also: newCopyWithNewUniqueIdentifier()
+ *
  * @param obj
  *    Data copied from obj to this.
  * @return 
@@ -85,6 +92,22 @@ TileTabsConfiguration::operator=(const TileTabsConfiguration& obj)
     return *this;    
 }
 
+/**
+ * Copy this instance and give it a new unique identifier.
+ * Note that copy constructor does not create a new unique identifier.
+ *
+ * @return The new Copy.
+ */
+TileTabsConfiguration*
+TileTabsConfiguration::newCopyWithNewUniqueIdentifier() const
+{
+    TileTabsConfiguration* newCopy = new TileTabsConfiguration(*this);
+    CaretAssert(newCopy);
+    newCopy->m_uniqueIdentifier = SystemUtilities::createUniqueID();
+    return newCopy;
+}
+
+
 /**
  * Initialize an instance of a tile tabs configuration.
  */
@@ -95,12 +118,12 @@ TileTabsConfiguration::initialize()
                                1.0);
     m_columnStretchFactors.resize(getMaximumNumberOfColumns(),
                                   1.0);
-
+    m_numberOfColumns = 0;
+    m_numberOfRows    = 0;
+    
     setNumberOfRows(2);
     setNumberOfColumns(2);
     
-    m_defaultConfigurationFlag = false;
-
     m_uniqueIdentifier = SystemUtilities::createUniqueID();
 }
 
@@ -112,6 +135,10 @@ TileTabsConfiguration::initialize()
 void 
 TileTabsConfiguration::copyHelperTileTabsConfiguration(const TileTabsConfiguration& obj)
 {
+    if (this == &obj) {
+        return;
+    }
+    
     m_numberOfColumns = obj.m_numberOfColumns;
     m_numberOfRows    = obj.m_numberOfRows;
     m_rowStretchFactors = obj.m_rowStretchFactors;
@@ -120,6 +147,18 @@ TileTabsConfiguration::copyHelperTileTabsConfiguration(const TileTabsConfigurati
     //DO NOT CHANGE THE UNIQUE IDENTIFIER:  m_uniqueIdentifier
 }
 
+/**
+ * Copies the tile tabs configuration rows, columns, and
+ * stretch factors.   Name is NOT copied.
+ */
+void
+TileTabsConfiguration::copy(const TileTabsConfiguration& rhs)
+{
+    AString savedName = m_name;
+    copyHelperTileTabsConfiguration(rhs);
+    m_name = savedName;
+}
+
 /**
  * Get the row heights and column widths for this tile tabs configuration using the
  * given window width and height.
@@ -130,6 +169,8 @@ TileTabsConfiguration::copyHelperTileTabsConfiguration(const TileTabsConfigurati
  *      Height of window.
  * @param numberOfModelsToDraw
  *      Number of models to draw.
+ * @param configurationMode
+ *      The tile tabs configuration mode
  * @param rowHeightsOut
  *      Output containing height of each row.
  * @param columnWidthsOut
@@ -141,6 +182,7 @@ bool
 TileTabsConfiguration::getRowHeightsAndColumnWidthsForWindowSize(const int32_t windowWidth,
                                                                  const int32_t windowHeight,
                                                                  const int32_t numberOfModelsToDraw,
+                                                                 const TileTabsConfigurationModeEnum::Enum configurationMode,
                                                                  std::vector<int32_t>& rowHeightsOut,
                                                                  std::vector<int32_t>& columnWidthsOut)
 {
@@ -158,58 +200,62 @@ TileTabsConfiguration::getRowHeightsAndColumnWidthsForWindowSize(const int32_t w
     rowHeightsOut.clear();
     columnWidthsOut.clear();
     
-    if (isDefaultConfiguration()) {
-        /*
-         * Update number of rows/columns in the default configuration
-         * so that if a scene is saved, the correct number of rows
-         * and columns are saved to the scene.
-         */
-        updateDefaultConfigurationRowsAndColumns(numberOfModelsToDraw);
-        numRows = getNumberOfRows();
-        numCols = getNumberOfColumns();
-        
-        for (int32_t i = 0; i < numRows; i++) {
-            rowHeightsOut.push_back(windowHeight / numRows);
-        }
-        for (int32_t i = 0; i < numCols; i++) {
-            columnWidthsOut.push_back(windowWidth / numCols);
-        }
-    }
-    else {
-        /*
-         * Rows/columns from user configuration
-         */
-        numRows = getNumberOfRows();
-        numCols = getNumberOfColumns();
-        
-        /*
-         * Determine height of each row
-         */
-        float rowStretchTotal = 0.0;
-        for (int32_t i = 0; i < numRows; i++) {
-            rowStretchTotal += getRowStretchFactor(i);
-        }
-        CaretAssert(rowStretchTotal > 0.0);
-        for (int32_t i = 0; i < numRows; i++) {
-            const int32_t h = static_cast<int32_t>((getRowStretchFactor(i) / rowStretchTotal)
-                                                   * windowHeight);
+    switch (configurationMode) {
+        case TileTabsConfigurationModeEnum::AUTOMATIC:
+        {
+            /*
+             * Update number of rows/columns in the default configuration
+             * so that if a scene is saved, the correct number of rows
+             * and columns are saved to the scene.
+             */
+            updateAutomaticConfigurationRowsAndColumns(numberOfModelsToDraw);
+            numRows = getNumberOfRows();
+            numCols = getNumberOfColumns();
             
-            rowHeightsOut.push_back(h);
-        }
-        
-        /*
-         * Determine width of each column
-         */
-        float columnStretchTotal = 0.0;
-        for (int32_t i = 0; i < numCols; i++) {
-            columnStretchTotal += getColumnStretchFactor(i);
-        }
-        CaretAssert(columnStretchTotal > 0.0);
-        for (int32_t i = 0; i < numCols; i++) {
-            const int32_t w = static_cast<int32_t>((getColumnStretchFactor(i) / columnStretchTotal)
-                                                   * windowWidth);
-            columnWidthsOut.push_back(w);
-        }
+            for (int32_t i = 0; i < numRows; i++) {
+                rowHeightsOut.push_back(windowHeight / numRows);
+            }
+            for (int32_t i = 0; i < numCols; i++) {
+                columnWidthsOut.push_back(windowWidth / numCols);
+            }
+        }            break;
+        case TileTabsConfigurationModeEnum::CUSTOM:
+        {
+            /*
+             * Rows/columns from user configuration
+             */
+            numRows = getNumberOfRows();
+            numCols = getNumberOfColumns();
+            
+            /*
+             * Determine height of each row
+             */
+            float rowStretchTotal = 0.0;
+            for (int32_t i = 0; i < numRows; i++) {
+                rowStretchTotal += getRowStretchFactor(i);
+            }
+            CaretAssert(rowStretchTotal > 0.0);
+            for (int32_t i = 0; i < numRows; i++) {
+                const int32_t h = static_cast<int32_t>((getRowStretchFactor(i) / rowStretchTotal)
+                                                       * windowHeight);
+                
+                rowHeightsOut.push_back(h);
+            }
+            
+            /*
+             * Determine width of each column
+             */
+            float columnStretchTotal = 0.0;
+            for (int32_t i = 0; i < numCols; i++) {
+                columnStretchTotal += getColumnStretchFactor(i);
+            }
+            CaretAssert(columnStretchTotal > 0.0);
+            for (int32_t i = 0; i < numCols; i++) {
+                const int32_t w = static_cast<int32_t>((getColumnStretchFactor(i) / columnStretchTotal)
+                                                       * windowWidth);
+                columnWidthsOut.push_back(w);
+            }
+        }            break;
     }
     
     if ((numRows == static_cast<int32_t>(rowHeightsOut.size()))
@@ -307,8 +353,9 @@ TileTabsConfiguration::getNumberOfRows() const
 void
 TileTabsConfiguration::setNumberOfRows(const int32_t numberOfRows)
 {
-    CaretAssert(numberOfRows >= 1);
+    const int32_t oldNumerOfRows = m_numberOfRows;
     
+    CaretAssert(numberOfRows >= 1);
     m_numberOfRows = numberOfRows;
     if (m_numberOfRows > getMaximumNumberOfRows()) {
         CaretLogSevere("Requested number of rows is "
@@ -317,6 +364,14 @@ TileTabsConfiguration::setNumberOfRows(const int32_t numberOfRows)
                        + getMaximumNumberOfRows());
         m_numberOfRows = getMaximumNumberOfRows();
     }
+    
+    /*
+     * Stretch factors default to 1.0
+     */
+    for (int32_t iRow = oldNumerOfRows; iRow < m_numberOfRows; iRow++) {
+        CaretAssertVectorIndex(m_rowStretchFactors, iRow);
+        m_rowStretchFactors[iRow] = 1.0;
+    }
 }
 
 /**
@@ -337,6 +392,8 @@ TileTabsConfiguration::getNumberOfColumns() const
 void
 TileTabsConfiguration::setNumberOfColumns(const int32_t numberOfColumns)
 {
+    const int32_t oldNumberOfColumns = m_numberOfColumns;
+    
     CaretAssert(numberOfColumns >= 1);
     
     m_numberOfColumns = numberOfColumns;
@@ -347,6 +404,14 @@ TileTabsConfiguration::setNumberOfColumns(const int32_t numberOfColumns)
                        + getMaximumNumberOfColumns());
         m_numberOfColumns = getMaximumNumberOfColumns();
     }
+    
+    /*
+     * Stretch factors default to 1.0
+     */
+    for (int32_t iCol = oldNumberOfColumns; iCol < m_numberOfColumns; iCol++) {
+        CaretAssertVectorIndex(m_columnStretchFactors, iCol);
+        m_columnStretchFactors[iCol] = 1.0;
+    }
 }
 
 /**
@@ -417,51 +482,57 @@ TileTabsConfiguration::setRowStretchFactor(const int32_t rowIndex,
 }
 
 /**
- * @return Is this the default configuration?  Each browser window
- * always has ONE default configuration which displays all tabs.
- */
-bool
-TileTabsConfiguration::isDefaultConfiguration() const
-{
-    return m_defaultConfigurationFlag;
-}
-
-/**
- * Set the default configuration status.  This should only be called by
- * the browser window which contains one tile tabs configuration that is
- * used by the browser window for display of all tabs.
+ * Get the number of rows and columns for an automatic layout with the
+ * given number of tabs.
+ * @param numberOfTabs
+ *     Number of tabs.
+ * @param numberOfRowsOut
+ *     Output with number of rows.
+ * @param numberOfColumnsOut
+ *     Output with number of columns.
  */
 void
-TileTabsConfiguration::setDefaultConfiguration(const bool defaultConfiguration)
+TileTabsConfiguration::getRowsAndColumnsForNumberOfTabs(const int32_t numberOfTabs,
+                                                        int32_t& numberOfRowsOut,
+                                                        int32_t& numberOfColumnsOut)
 {
-    m_defaultConfigurationFlag = defaultConfiguration;
+    CaretAssert(numberOfTabs >= 0);
+    
+    numberOfRowsOut = (int)std::sqrt((double)numberOfTabs);
+    numberOfColumnsOut = numberOfRowsOut;
+    int32_t row2 = numberOfRowsOut * numberOfRowsOut;
+    if (row2 < numberOfTabs) {
+        numberOfColumnsOut++;
+    }
+    if ((numberOfRowsOut * numberOfColumnsOut) < numberOfTabs) {
+        numberOfRowsOut++;
+    }
 }
 
 /**
- * Updates the number of rows and columns in this default configuration
- * based upon the number of tabs.  If this is NOT the default configuration
- * (isDefaultConfiguration() returns false), no action is taken.
+ * Updates the number of rows and columns for the automatic configuration
+ * based upon the number of tabs.  
  *
  * Since screen width typically exceeds height, ensure the number of 
  * columns is always greater than the number of rows.
  */
 void
-TileTabsConfiguration::updateDefaultConfigurationRowsAndColumns(const int32_t numberOfTabs)
+TileTabsConfiguration::updateAutomaticConfigurationRowsAndColumns(const int32_t numberOfTabs)
 {
-    if (isDefaultConfiguration()) {
-        int32_t numRows = (int)std::sqrt((double)numberOfTabs);
-        int32_t numCols = numRows;
-        int32_t row2 = numRows * numRows;
-        if (row2 < numberOfTabs) {
-            numCols++;
-        }
-        if ((numRows * numCols) < numberOfTabs) {
-            numRows++;
-        }
-        
-        setNumberOfRows(numRows);
-        setNumberOfColumns(numCols);
-    }
+    int32_t numRows(0), numCols(0);
+    getRowsAndColumnsForNumberOfTabs(numberOfTabs,
+                                     numRows,
+                                     numCols);
+    
+    setNumberOfRows(numRows);
+    setNumberOfColumns(numCols);
+    
+    std::fill(m_columnStretchFactors.begin(),
+              m_columnStretchFactors.end(),
+              1.0);
+    std::fill(m_rowStretchFactors.begin(),
+              m_rowStretchFactors.end(),
+              1.0);
 }
 
 
@@ -521,7 +592,6 @@ TileTabsConfiguration::encodeInXML() const
 bool
 TileTabsConfiguration::decodeFromXML(const AString& xmlString)
 {
-    m_defaultConfigurationFlag = false;
     setNumberOfRows(2);
     setNumberOfColumns(2);
     
