@@ -36,11 +36,14 @@
 #include "CaretAssert.h"
 #include "CaretColorEnum.h"
 #include "CaretLogger.h"
+#include "DataFileContentCopyMoveParameters.h"
 #include "DataFileException.h"
 #include "DisplayGroupAndTabItemHelper.h"
 #include "EventAnnotationAddToRemoveFromFile.h"
 #include "EventAnnotationGroupGetWithKey.h"
 #include "EventAnnotationGrouping.h"
+#include "EventAnnotationTextSubstitutionInvalidate.h"
+#include "EventBrowserTabDelete.h"
 #include "EventManager.h"
 #include "GiftiMetaData.h"
 #include "SceneClass.h"
@@ -217,6 +220,8 @@ AnnotationFile::initializeAnnotationFile()
     EventManager::get()->addEventListener(this, EventTypeEnum::EVENT_ANNOTATION_ADD_TO_REMOVE_FROM_FILE);
     EventManager::get()->addEventListener(this, EventTypeEnum::EVENT_ANNOTATION_GROUP_GET_WITH_KEY);
     EventManager::get()->addEventListener(this, EventTypeEnum::EVENT_ANNOTATION_GROUPING);
+    EventManager::get()->addEventListener(this, EventTypeEnum::EVENT_ANNOTATION_TEXT_SUBSTITUTION_INVALIDATE);
+    EventManager::get()->addEventListener(this, EventTypeEnum::EVENT_BROWSER_TAB_DELETE);
 }
 
 /**
@@ -234,7 +239,7 @@ AnnotationFile::getSpaceAnnotationGroup(const Annotation* annotation)
     const AnnotationCoordinateSpaceEnum::Enum annotationSpace = annotation->getCoordinateSpace();
     int32_t annotationTabOrWindowIndex = -1;
     switch (annotationSpace) {
-        case AnnotationCoordinateSpaceEnum::PIXELS:
+        case AnnotationCoordinateSpaceEnum::CHART:
             break;
         case AnnotationCoordinateSpaceEnum::STEREOTAXIC:
             break;
@@ -243,6 +248,8 @@ AnnotationFile::getSpaceAnnotationGroup(const Annotation* annotation)
         case AnnotationCoordinateSpaceEnum::TAB:
             annotationTabOrWindowIndex = annotation->getTabIndex();
             break;
+        case AnnotationCoordinateSpaceEnum::VIEWPORT:
+            break;
         case AnnotationCoordinateSpaceEnum::WINDOW:
             annotationTabOrWindowIndex = annotation->getWindowIndex();
             break;
@@ -255,13 +262,14 @@ AnnotationFile::getSpaceAnnotationGroup(const Annotation* annotation)
         if (group->getGroupType() == AnnotationGroupTypeEnum::SPACE) {
             if (group->getCoordinateSpace() == annotationSpace) {
                 switch (annotationSpace) {
-                    case AnnotationCoordinateSpaceEnum::PIXELS:
-                        CaretAssert(0);
-                        break;
+                    case AnnotationCoordinateSpaceEnum::CHART:
                     case AnnotationCoordinateSpaceEnum::STEREOTAXIC:
                     case AnnotationCoordinateSpaceEnum::SURFACE:
                         return group;
                         break;
+                    case AnnotationCoordinateSpaceEnum::VIEWPORT:
+                        CaretAssert(0);
+                        break;
                     case AnnotationCoordinateSpaceEnum::TAB:
                     case AnnotationCoordinateSpaceEnum::WINDOW:
                         if (annotationTabOrWindowIndex == group->getTabOrWindowIndex()) {
@@ -274,9 +282,7 @@ AnnotationFile::getSpaceAnnotationGroup(const Annotation* annotation)
     }
     
     switch (annotationSpace) {
-        case AnnotationCoordinateSpaceEnum::PIXELS:
-            CaretAssert(0);
-            break;
+        case AnnotationCoordinateSpaceEnum::CHART:
         case AnnotationCoordinateSpaceEnum::STEREOTAXIC:
         case AnnotationCoordinateSpaceEnum::SURFACE:
             break;
@@ -284,6 +290,9 @@ AnnotationFile::getSpaceAnnotationGroup(const Annotation* annotation)
             CaretAssert((annotationTabOrWindowIndex >= 0)
                         && (annotationTabOrWindowIndex < BrainConstants::MAXIMUM_NUMBER_OF_BROWSER_TABS));
             break;
+        case AnnotationCoordinateSpaceEnum::VIEWPORT:
+            CaretAssert(0);
+            break;
         case AnnotationCoordinateSpaceEnum::WINDOW:
             CaretAssert((annotationTabOrWindowIndex >= 0)
                         && (annotationTabOrWindowIndex < BrainConstants::MAXIMUM_NUMBER_OF_BROWSER_WINDOWS));
@@ -340,15 +349,22 @@ AnnotationFile::receiveEvent(Event* event)
                 }
                 break;
             case EventAnnotationAddToRemoveFromFile::MODE_CUT:
-                if (removeAnnotation(annotation)) {
+                if (removeAnnotationWithUndoRedo(annotation)) {
                     annEvent->setSuccessful(true);
                 }
                 break;
             case EventAnnotationAddToRemoveFromFile::MODE_DELETE:
-                if (removeAnnotation(annotation)) {
+                if (removeAnnotationWithUndoRedo(annotation)) {
                     annEvent->setSuccessful(true);
                 }
                 break;
+            case EventAnnotationAddToRemoveFromFile::MODE_DUPLICATE:
+                if (annotationFile == this) {
+                    if (restoreAnnotationAddIfNotFound(annotation)) {
+                        annEvent->setSuccessful(true);
+                    }
+                }
+                break;
             case EventAnnotationAddToRemoveFromFile::MODE_PASTE:
                 if (annotationFile == this) {
                     if (restoreAnnotationAddIfNotFound(annotation)) {
@@ -358,7 +374,7 @@ AnnotationFile::receiveEvent(Event* event)
                 break;
             case EventAnnotationAddToRemoveFromFile::MODE_UNCREATE:
                 if (annotationFile == this) {
-                    if (removeAnnotation(annotation)) {
+                    if (removeAnnotationWithUndoRedo(annotation)) {
                         annEvent->setSuccessful(true);
                     }
                 }
@@ -373,9 +389,15 @@ AnnotationFile::receiveEvent(Event* event)
                     annEvent->setSuccessful(true);
                 }
                 break;
+            case EventAnnotationAddToRemoveFromFile::MODE_UNDUPLICATE:
+                if (annotationFile == this) {
+                    if (removeAnnotationWithUndoRedo(annotation)) {
+                        annEvent->setSuccessful(true);
+                    }
+                }
             case EventAnnotationAddToRemoveFromFile::MODE_UNPASTE:
                 if (annotationFile == this) {
-                    if (removeAnnotation(annotation)) {
+                    if (removeAnnotationWithUndoRedo(annotation)) {
                         annEvent->setSuccessful(true);
                     }
                 }
@@ -418,6 +440,30 @@ AnnotationFile::receiveEvent(Event* event)
             }
         }
     }
+    else if (event->getEventType() == EventTypeEnum::EVENT_BROWSER_TAB_DELETE) {
+        EventBrowserTabDelete* deleteEvent = dynamic_cast<EventBrowserTabDelete*>(event);
+        CaretAssert(deleteEvent);
+        
+        const int32_t tabIndex = deleteEvent->getBrowserTabIndex();
+        if (removeAnnotationsInTab(tabIndex)) {
+            deleteEvent->setEventProcessed();
+        }
+    }
+    else if (event->getEventType() == EventTypeEnum::EVENT_ANNOTATION_TEXT_SUBSTITUTION_INVALIDATE) {
+        EventAnnotationTextSubstitutionInvalidate* textSubEvent = dynamic_cast<EventAnnotationTextSubstitutionInvalidate*>(event);
+        CaretAssert(textSubEvent);
+        
+        for (auto ag : m_annotationGroups) {
+            std::vector<Annotation*> annotations;
+            ag->getAllAnnotations(annotations);
+            
+            for (auto ann : annotations) {
+                ann->invalidateTextSubstitution();
+            }
+        }
+        
+        textSubEvent->setEventProcessed();
+    }
 }
 
 /**
@@ -606,13 +652,15 @@ AnnotationFile::addAnnotationGroupDuringFileReading(const AnnotationGroupTypeEnu
     }
     
     switch (coordinateSpace) {
-        case AnnotationCoordinateSpaceEnum::PIXELS:
-            throw DataFileException("PIXELS coordinate space is not allowed for group while annotation file.");
+        case AnnotationCoordinateSpaceEnum::CHART:
             break;
         case AnnotationCoordinateSpaceEnum::STEREOTAXIC:
             break;
         case AnnotationCoordinateSpaceEnum::SURFACE:
             break;
+        case AnnotationCoordinateSpaceEnum::VIEWPORT:
+            throw DataFileException("VIEWPORT coordinate space is not allowed for group while annotation file.");
+            break;
         case AnnotationCoordinateSpaceEnum::TAB:
         case AnnotationCoordinateSpaceEnum::WINDOW:
             if ((tabOrWindowIndex < 0)
@@ -652,9 +700,7 @@ AnnotationFile::addAnnotationGroupDuringFileReading(const AnnotationGroupTypeEnu
             if (groupType == AnnotationGroupTypeEnum::SPACE) {
                 if (group->getCoordinateSpace() == coordinateSpace) {
                     switch (coordinateSpace) {
-                        case AnnotationCoordinateSpaceEnum::PIXELS:
-                            CaretAssert(0);
-                            break;
+                        case AnnotationCoordinateSpaceEnum::CHART:
                         case AnnotationCoordinateSpaceEnum::STEREOTAXIC:
                         case AnnotationCoordinateSpaceEnum::SURFACE:
                             throw DataFileException("There is more than one annotation space group with space "
@@ -670,6 +716,9 @@ AnnotationFile::addAnnotationGroupDuringFileReading(const AnnotationGroupTypeEnu
                                                         + ".  Only one space group for each space is allowed.");
                             }
                             break;
+                        case AnnotationCoordinateSpaceEnum::VIEWPORT:
+                            CaretAssert(0);
+                            break;
                         case AnnotationCoordinateSpaceEnum::WINDOW:
                             if (tabOrWindowIndex == group->getTabOrWindowIndex()) {
                                 throw DataFileException("There is more than one annotation space group with space "
@@ -761,9 +810,9 @@ AnnotationFile::restoreAnnotation(Annotation* annotation)
 }
 
 /**
- * Remove the annotation.  NOTE: The annotation is NOT deleted
- * but instead it is saved so that it can be 'undeleted' 
- * or 're-pasted'.
+ * Remove the annotation for use with REDO/UNDO.  
+ * NOTE: The annotation is NOT deleted but instead
+ * it is saved so that it can be 'undeleted' or 're-pasted'.
  *
  * @param annotation
  *     Annotation that is removed.
@@ -771,9 +820,56 @@ AnnotationFile::restoreAnnotation(Annotation* annotation)
  *     True if the annotation was removed, otherwise false.
  */
 bool
-AnnotationFile::removeAnnotation(Annotation* annotation)
+AnnotationFile::removeAnnotationWithUndoRedo(Annotation* annotation)
 {
+    return removeAnnotationPrivate(annotation, true);
     
+    
+//    for (AnnotationGroupIterator groupIter = m_annotationGroups.begin();
+//         groupIter != m_annotationGroups.end();
+//         groupIter++) {
+//        QSharedPointer<AnnotationGroup> group = *groupIter;
+//        QSharedPointer<Annotation> removedAnnotationPointer;
+//        if (group->removeAnnotation(annotation,
+//                                    removedAnnotationPointer)) {
+//            
+//            removedAnnotationPointer->invalidateAnnotationGroupKey();
+//            
+//            m_removedAnnotations.insert(removedAnnotationPointer);
+//            
+//            /*
+//             * Remove group if it is empty.
+//             */
+//            if (group->isEmpty()) {
+//                m_annotationGroups.erase(groupIter);
+//            }
+//            
+//            setModified();
+//            return true;
+//        }
+//    }
+//    
+//    /*
+//     * Annotation not in this file
+//     */
+//    return false;
+}
+
+/**
+ * Remove the 
+ *
+ * @param annotation
+ *     Annotation that is removed.
+ * @param keepAnnotationForUndoRedoFlag
+ *     If true, keep the annotation available for any
+ *     undo or redo operation.
+ * @return
+ *     True if the annotation was removed, otherwise false.
+ */
+bool
+AnnotationFile::removeAnnotationPrivate(Annotation* annotation,
+                                        const bool keepAnnotationForUndoRedoFlag)
+{
     for (AnnotationGroupIterator groupIter = m_annotationGroups.begin();
          groupIter != m_annotationGroups.end();
          groupIter++) {
@@ -784,7 +880,9 @@ AnnotationFile::removeAnnotation(Annotation* annotation)
             
             removedAnnotationPointer->invalidateAnnotationGroupKey();
             
-            m_removedAnnotations.insert(removedAnnotationPointer);
+            if (keepAnnotationForUndoRedoFlag) {
+                m_removedAnnotations.insert(removedAnnotationPointer);
+            }
             
             /*
              * Remove group if it is empty.
@@ -797,23 +895,6 @@ AnnotationFile::removeAnnotation(Annotation* annotation)
             return true;
         }
     }
-//    for (AnnotationIterator iter = m_annotations.begin();
-//         iter != m_annotations.end();
-//         iter++) {
-//        QSharedPointer<Annotation>& annotationPointer = *iter;
-//        if (annotationPointer == annotation) {
-//            m_removedAnnotations.insert(annotationPointer);
-//            
-//            m_annotations.erase(iter);
-//            
-//            setModified();
-//            
-//            /*
-//             * Successfully removed
-//             */
-//            return true;
-//        }
-//    }
     
     /*
      * Annotation not in this file
@@ -821,6 +902,50 @@ AnnotationFile::removeAnnotation(Annotation* annotation)
     return false;
 }
 
+/**
+ * Remove all annotations in tab space in the given tab.
+ *
+ * @param tabIndex
+ *     Index of the tab.
+ * @return
+ *     True if any annotations were removed.
+ */
+bool
+AnnotationFile::removeAnnotationsInTab(const int32_t tabIndex)
+{
+    std::vector<AnnotationGroupIterator> groupsToRemoveIterators;
+    
+    /*
+     * Find annotation group(s) in tab space
+     * with the given tab index.
+     */
+    for (AnnotationGroupIterator groupIter = m_annotationGroups.begin();
+         groupIter != m_annotationGroups.end();
+         groupIter++) {
+        QSharedPointer<AnnotationGroup>& group = *groupIter;
+        if (group->getCoordinateSpace() == AnnotationCoordinateSpaceEnum::TAB) {
+            if (group->getTabOrWindowIndex() == tabIndex) {
+                groupsToRemoveIterators.push_back(groupIter);
+            }
+        }
+    }
+    
+    /*
+     * Remove the groups (shared pointers will do all deleting)
+     * NOTE: MUST delete iterators that are closest to the end first
+     * as any iterators that point to elements at the erased iterator
+     * or beyond are invalidated.
+     */
+    const int32_t numGroupsToRemove = static_cast<int32_t>(groupsToRemoveIterators.size());
+    for (int32_t i = (numGroupsToRemove - 1); i >= 0; i--) {
+        CaretAssertVectorIndex(groupsToRemoveIterators, i);
+        m_annotationGroups.erase(groupsToRemoveIterators[i]);
+    }
+    
+    return ( ! groupsToRemoveIterators.empty());
+}
+
+
 /**
  * Get all annotations in this file.
  * 
@@ -1138,6 +1263,17 @@ AnnotationFile::processRegroupingAnnotations(EventAnnotationGrouping* groupingEv
                 }
             }
             
+            if ( ! allValidFlag) {
+                for (std::vector<QSharedPointer<Annotation> >::iterator annPtrIter = movedAnnotations.begin();
+                     annPtrIter != movedAnnotations.end();
+                     annPtrIter++) {
+                    spaceGroup->addAnnotationPrivateSharedPointer(*annPtrIter);
+                }
+                
+                groupingEvent->setErrorMessage("PROGRAM ERROR: Failed to remove an anntotation from its space group.");
+                return;
+            }
+            
             for (std::vector<QSharedPointer<Annotation> >::iterator annPtrIter = movedAnnotations.begin();
                  annPtrIter != movedAnnotations.end();
                  annPtrIter++) {
@@ -1409,6 +1545,10 @@ AnnotationFile::readFile(const AString& filename)
 void
 AnnotationFile::writeFile(const AString& filename)
 {
+    if (!(filename.endsWith(".annot") || filename.endsWith(".wb_annot")))
+    {
+        CaretLogWarning("annotation file '" + filename + "' should be saved ending in .annot");
+    }
     checkFileWritability(filename);
     
     setFileName(filename);
@@ -1541,20 +1681,28 @@ AnnotationFile::getAsDataFile()
 /**
  * Append content from the given data file copy/move interface to this instance
  *
- * @param dataFileCopyMoveInterface
- *     From which content is copied.
+ * @param copyMoveParameters
+ *     Parameters used for copy/move.
  * @throws DataFileException
  *     If there is an error.
  */
 void
-AnnotationFile::appendContentFromDataFile(const DataFileContentCopyMoveInterface* dataFileCopyMoveInterface)
+AnnotationFile::appendContentFromDataFile(const DataFileContentCopyMoveParameters& copyMoveParameters)
 {
-    const AnnotationFile* copyFromFile = dynamic_cast<const AnnotationFile*>(dataFileCopyMoveInterface);
+    const AnnotationFile* copyFromFile = dynamic_cast<const AnnotationFile*>(copyMoveParameters.getDataFileCopyMoveInterfaceToCopy());
     if (copyFromFile == NULL) {
         throw DataFileException("Trying to copy content to annotation file from a file that is not an "
                                 "annotation file.");
     }
     
+    const bool selectedAnnotationsFlag = copyMoveParameters.isOptionSelectedItems();
+    const int32_t windowIndex = copyMoveParameters.getWindowIndex();
+    if (selectedAnnotationsFlag) {
+        if (windowIndex < 0) {
+            throw DataFileException("Requested copying of selected annotations but window index is invalid.");
+        }
+    }
+    
     std::vector<AnnotationGroup*> annotationGroups;
     copyFromFile->getAllAnnotationGroups(annotationGroups);
     
@@ -1563,9 +1711,36 @@ AnnotationFile::appendContentFromDataFile(const DataFileContentCopyMoveInterface
          groupIter++) {
         const AnnotationGroup* groupToCopy = *groupIter;
         CaretAssert(groupToCopy);
+
+        /*
+         * Find that annotations that are to be copied.
+         */
+        const int32_t numAnnInGroup = groupToCopy->getNumberOfAnnotations();
+        std::vector<const Annotation*> annotationsToCopy;
+        for (int32_t ia = 0; ia < numAnnInGroup; ia++) {
+            const Annotation* annToCopy = groupToCopy->getAnnotation(ia);
+            CaretAssert(annToCopy);
+            
+            bool copyFlag = true;
+            if (selectedAnnotationsFlag) {
+                if (annToCopy->isSelectedForEditing(windowIndex)) {
+                    /* Nothing */
+                }
+                else {
+                    copyFlag = false;
+                }
+            }
+            
+            if (copyFlag) {
+                annotationsToCopy.push_back(annToCopy);
+            }
+        }
         
-        const int32_t numAnnInGroupToCopy = groupToCopy->getNumberOfAnnotations();
-        if (numAnnInGroupToCopy > 0) {
+        /*
+         * If there are are annotations to copy,
+         * clone them and add to the proper group.
+         */
+        if ( ! annotationsToCopy.empty()) {
             AnnotationGroup* group = NULL;
             switch (groupToCopy->getGroupType()) {
                 case AnnotationGroupTypeEnum::INVALID:
@@ -1596,11 +1771,13 @@ AnnotationFile::appendContentFromDataFile(const DataFileContentCopyMoveInterface
             /*
              * Copy annotations and add them to the group.
              */
-            for (int32_t ia = 0; ia < numAnnInGroupToCopy; ia++) {
-                const Annotation* annToCopy = groupToCopy->getAnnotation(ia);
-                CaretAssert(annToCopy);
-                Annotation* ann = annToCopy->clone();
-                group->addAnnotationPrivate(ann);
+            for (std::vector<const Annotation*>::const_iterator iter = annotationsToCopy.begin();
+                 iter != annotationsToCopy.end();
+                 iter++) {
+                const Annotation* annToCopy = *iter;
+                Annotation* clonedAnn = annToCopy->clone();
+                clonedAnn->setUniqueKey(generateUniqueKey());
+                group->addAnnotationPrivate(clonedAnn);
             }
         }
     }
