@@ -28,6 +28,7 @@
 
 #include "CaretAssert.h"
 #include "CaretLogger.h"
+#include "ControlPointFile.h"
 #include "ControlPoint3D.h"
 #include "DataFileException.h"
 #include "FileInformation.h"
@@ -36,7 +37,6 @@
 #include "ImageFile.h"
 #include "Matrix4x4.h"
 #include "MathFunctions.h"
-#include "PaletteFile.h"
 #include "SceneClass.h"
 #include "VolumeFile.h"
 
@@ -50,10 +50,11 @@ const float ImageFile::s_defaultWindowDepthPercentage = 990;
 ImageFile::ImageFile()
 : CaretDataFile(DataFileTypeEnum::IMAGE)
 {
+    m_controlPointFile.grabNew(new ControlPointFile());
+    
     m_fileMetaData.grabNew(new GiftiMetaData());
     
     m_image   = new QImage();
-    m_windowZ = s_defaultWindowDepthPercentage;
 }
 
 /**
@@ -64,10 +65,11 @@ ImageFile::ImageFile()
 ImageFile::ImageFile(const QImage& qimage)
 : CaretDataFile(DataFileTypeEnum::IMAGE)
 {
+    m_controlPointFile.grabNew(new ControlPointFile());
+    
     m_fileMetaData.grabNew(new GiftiMetaData());
     
     m_image = new QImage(qimage);
-    m_windowZ = s_defaultWindowDepthPercentage;
 }
 
 /**
@@ -89,12 +91,13 @@ ImageFile::ImageFile(const unsigned char* imageDataRGBA,
                      const IMAGE_DATA_ORIGIN_LOCATION imageOrigin)
 : CaretDataFile(DataFileTypeEnum::IMAGE)
 {
+    m_controlPointFile.grabNew(new ControlPointFile());
+    
     m_fileMetaData.grabNew(new GiftiMetaData());
     
     m_image = new QImage(imageWidth,
                              imageHeight,
                              QImage::Format_RGB32);
-    m_windowZ = s_defaultWindowDepthPercentage;
     
     bool isOriginAtTop = false;
     switch (imageOrigin) {
@@ -875,7 +878,7 @@ ImageFile::writeFile(const AString& filename)
         format = "JPEG";
     }
     
-    QImageWriter writer(filename, format.toAscii());
+    QImageWriter writer(filename, format.toLatin1());
     if (writer.supportsOption(QImageIOHandler::Quality)) {
         if (format.compare("png", Qt::CaseInsensitive) == 0) {
             const int quality = 1;
@@ -1176,6 +1179,79 @@ ImageFile::getImageBytesRGBA(const IMAGE_DATA_ORIGIN_LOCATION imageOrigin,
     return false;
 }
 
+/**
+ * Get the pixel RGBA at the given pixel I and J.
+ *
+ * @param imageOrigin
+ *    Location of first pixel in the image data.
+ * @param pixelI
+ *     Image I index
+ * @param pixelJ
+ *     Image J index
+ * @param pixelRGBAOut
+ *     RGBA at Pixel I, J
+ * @return
+ *     True if valid, else false.
+ */
+bool
+ImageFile::getImagePixelRGBA(const IMAGE_DATA_ORIGIN_LOCATION imageOrigin,
+                             const int32_t pixelI,
+                             const int32_t pixelJ,
+                             uint8_t pixelRGBAOut[4]) const
+{
+    if (m_image != NULL) {
+        const int32_t w = m_image->width();
+        const int32_t h = m_image->height();
+        
+        if ((pixelI >= 0)
+            && (pixelI < w)
+            && (pixelJ >= 0)
+            && (pixelJ < h)) {
+            
+            int32_t imageJ = pixelJ;
+            switch (imageOrigin) {
+                case IMAGE_DATA_ORIGIN_AT_BOTTOM:
+                    imageJ = h - pixelJ - 1;
+                    break;
+                case IMAGE_DATA_ORIGIN_AT_TOP:
+                    break;
+            }
+            
+            if ((imageJ >= 0)
+                && (imageJ < h)) {
+                const QRgb rgb = m_image->pixel(pixelI,
+                                                imageJ);
+                pixelRGBAOut[0] = static_cast<uint8_t>(qRed(rgb));
+                pixelRGBAOut[1] = static_cast<uint8_t>(qGreen(rgb));
+                pixelRGBAOut[2] = static_cast<uint8_t>(qBlue(rgb));
+                pixelRGBAOut[3] = 255;
+                
+                return true;
+            }
+            else {
+                CaretLogSevere("Invalid image J");
+            }
+        }
+    }
+    
+    return false;
+}
+
+
+/**
+ * Get the RGBA bytes from the image resized into the given width and height.
+ *
+ * @param imageOrigin
+ *    Location of first pixel in the image data.
+ * @param resizeToWidth
+ *    New width for image.
+ * @param resizeToHeight
+ *    New height of the image.
+ * @param bytesRGBAOut
+ *    The RGBA bytes in the image.
+ * @return
+ *    True if the bytes, width, and height are valid, else false.
+ */
 bool
 ImageFile::getImageResizedBytes(const IMAGE_DATA_ORIGIN_LOCATION imageOrigin,
                                 const int32_t resizeToWidth,
@@ -1273,34 +1349,6 @@ ImageFile::getHeight() const
     return h;
 }
 
-/**
- * @return
- *    The Window Z for the image.
- *
- * Note:  1000 is at far clipping plane
- *       -1000 is at near clipping plane
- */
-float
-ImageFile::getWindowZ() const
-{
-    return m_windowZ;
-}
-
-/**
- * Set the Window Z for the image.
- *
- * Note:  1000 is at far clipping plane
- *       -1000 is at near clipping plane
- *
- * @param windowZ
- *     New window Z for drawing image.
- */
-void
-ImageFile::setWindowZ(const float windowZ)
-{
-    m_windowZ = windowZ;
-}
-
 /**
  * Essentially writes the image file to a byte array using the given format.
  *
@@ -1328,7 +1376,7 @@ ImageFile::getImageInByteArray(QByteArray& byteArrayOut,
         }
         else {
             successFlag = m_image->save(&buffer,
-                                        format.toAscii().data());
+                                        format.toLatin1().data());
         }
         
         if ( ! successFlag) {
@@ -1347,7 +1395,7 @@ ImageFile::getImageInByteArray(QByteArray& byteArrayOut,
  * @param format
  *    Format for the image (jpg, ppm, etc.) or empty if unknown.
  */
-void
+bool
 ImageFile::setImageFromByteArray(const QByteArray& byteArray,
                                  const AString& format)
 {
@@ -1357,24 +1405,24 @@ ImageFile::setImageFromByteArray(const QByteArray& byteArray,
     }
     else {
         successFlag = m_image->loadFromData(byteArray,
-                                            format.toAscii().data());
+                                            format.toLatin1().data());
     }
     
     if ( ! successFlag) {
-        throw DataFileException(getFileName(),
-                                "Failed to create image from byte array.");
+        CaretLogSevere(getFileName()
+                       + " Failed to create image from byte array.");
     }
+
+    return successFlag;
 }
 
 /**
- * Convert this image into a Volume File.
+ * Convert this image into a Volume File using the 
+ * encapsulated Control Point File whose matrix 
+ * must have been updated.
  *
  * @param colorMode
  *     Color mode for conversion.
- * @param sformMatrix
- *     Matrix used as NIFTI sform.
- * @param paletteFile
- *     Palette file used for coloring the voxels.
  * @param errorMessageOut
  *     Contains error message if conversion fails.
  * @return
@@ -1385,8 +1433,6 @@ ImageFile::setImageFromByteArray(const QByteArray& byteArray,
  */
 VolumeFile*
 ImageFile::convertToVolumeFile(const CONVERT_TO_VOLUME_COLOR_MODE colorMode,
-                               const std::vector<ControlPoint3D>& controlPointsIn,
-                               const PaletteFile* paletteFile,
                                AString& errorMessageOut) const
 {
     errorMessageOut.clear();
@@ -1395,75 +1441,44 @@ ImageFile::convertToVolumeFile(const CONVERT_TO_VOLUME_COLOR_MODE colorMode,
     int32_t width = 0;
     int32_t height = 0;
     getImageBytesRGBA(ImageFile::IMAGE_DATA_ORIGIN_AT_BOTTOM,
-                                   rgbaBytes,
-                                   width,
-                                   height);
+                      rgbaBytes,
+                      width,
+                      height);
     if ((width <= 0)
         || (height <= 0)) {
         errorMessageOut = "Image width and/or height is invalid.";
         return NULL;
     }
     
-    std::vector<ControlPoint3D> controlPoints = controlPointsIn;
-    const int32_t numControlPoints = static_cast<int32_t>(controlPoints.size());
-    if (numControlPoints < 3) {
-        errorMessageOut = "There must be at least three control points.";
-        return NULL;
-    }
-    
-    float pixelNormalVector[3];
-    ControlPoint3D::getSourceNormalVector(controlPoints, pixelNormalVector);
-    
-    std::cout << "Pixel Normal Vector: " << qPrintable(AString::fromNumbers(pixelNormalVector, 3, ",")) << std::endl;
-    
-    const float tinyValue = 0.00001;
-    if ((pixelNormalVector[2] < tinyValue)
-        && (pixelNormalVector[2] > -tinyValue)) {
-        errorMessageOut = "Control points need to be in a triangular shape; not a line";
-        return false;
-    }
-    
-    if (pixelNormalVector[2] < 0.0) {
-        std::cout << "Swapping coordinates so image normal vector is counter clockwise" << std::endl;
-        std::swap(controlPoints[0], controlPoints[2]);
-        ControlPoint3D::getSourceNormalVector(controlPoints, pixelNormalVector);
-        std::cout << "After Swapping Pixel Normal Vector: " << qPrintable(AString::fromNumbers(pixelNormalVector, 3, ",")) << std::endl;
-    }
-    
-    Matrix4x4 matrix;
-    if ( ! matrix.createLandmarkTransformMatrix(controlPoints,
-                                                errorMessageOut)) {
-        return NULL;
-    }
+    const Matrix4x4* transformationMatrix = m_controlPointFile->getLandmarkTransformationMatrix();
     
     float firstPixel[3] = { 0, 0, 0 };
-    matrix.multiplyPoint3(firstPixel);
+    transformationMatrix->multiplyPoint3(firstPixel);
     std::cout << "First pixel coord: " << AString::fromNumbers(firstPixel, 3, ",") << std::endl;
     
-    float lastPixel[3] = { width - 1, height - 1, 0 };
-    matrix.multiplyPoint3(lastPixel);
+    float lastPixel[3] = { (float)(width - 1), (float)(height - 1), 0.0f };
+    transformationMatrix->multiplyPoint3(lastPixel);
     std::cout << "Last pixel coord: " << AString::fromNumbers(lastPixel, 3, ",") << std::endl;
     
     {
         float bl[3] = { 0.0, 0.0, 0.0 };
-        matrix.multiplyPoint3(bl);
+        transformationMatrix->multiplyPoint3(bl);
         ControlPoint3D bottomLeft(0, 0, 0, bl[0], bl[1], bl[2]);
         
-        float br[3] = { width - 1.0, 0.0, 0.0 };
-        matrix.multiplyPoint3(br);
+        float br[3] = { (float)(width - 1.0), 0.0, 0.0 };
+        transformationMatrix->multiplyPoint3(br);
         ControlPoint3D bottomRight(width - 1.0, 0.0, 0.0, br[0], br[1], br[2]);
         
-        float tr[3] = { width - 1.0, height - 1.0, 0.0 };
-        matrix.multiplyPoint3(tr);
-        ControlPoint3D topRight(width - 1.0, height - 1.0, 0.0, tr[0], tr[1], tr[2]);
+        float tr[3] = { (float)(width - 1.0), (float)(height - 1.0), 0.0 };
+        transformationMatrix->multiplyPoint3(tr);
+        ControlPoint3D topRight((float)(width - 1.0), (float)(height - 1.0), 0.0, tr[0], tr[1], tr[2]);
         
-        std::vector<ControlPoint3D> volumeControlPoints;
-        volumeControlPoints.push_back(bottomLeft);
-        volumeControlPoints.push_back(bottomRight);
-        volumeControlPoints.push_back(topRight);
+        ControlPointFile volumeControlPointFile;
+        volumeControlPointFile.addControlPoint(bottomLeft);
+        volumeControlPointFile.addControlPoint(bottomRight);
+        volumeControlPointFile.addControlPoint(topRight);
         
-        Matrix4x4 volumeMatrix;
-        if ( ! volumeMatrix.createLandmarkTransformMatrix(volumeControlPoints, errorMessageOut)) {
+        if ( ! volumeControlPointFile.updateLandmarkTransformationMatrix(errorMessageOut)) {
             errorMessageOut.insert(0, "Volume Matrix: ");
             return NULL;
         }
@@ -1481,12 +1496,12 @@ ImageFile::convertToVolumeFile(const CONVERT_TO_VOLUME_COLOR_MODE colorMode,
     std::vector<float> row2;
     std::vector<float> row3;
     std::vector<float> row4;
-
+    
     for (int j = 0; j < 4; j++) {
-        row1.push_back(matrix.getMatrixElement(0, j));
-        row2.push_back(matrix.getMatrixElement(1, j));
-        row3.push_back(matrix.getMatrixElement(2, j));
-        row4.push_back(matrix.getMatrixElement(3, j));
+        row1.push_back(transformationMatrix->getMatrixElement(0, j));
+        row2.push_back(transformationMatrix->getMatrixElement(1, j));
+        row3.push_back(transformationMatrix->getMatrixElement(2, j));
+        row4.push_back(transformationMatrix->getMatrixElement(3, j));
     }
     std::vector<std::vector<float> > indexToSpace;
     indexToSpace.push_back(row1);
@@ -1504,11 +1519,10 @@ ImageFile::convertToVolumeFile(const CONVERT_TO_VOLUME_COLOR_MODE colorMode,
             whatType = SubvolumeAttributes::RGB;
             break;
     }
-
     VolumeFile* volumeFile = new VolumeFile(dimensions,
-                                    indexToSpace,
-                                    numComponents,
-                                    whatType);
+                                            indexToSpace,
+                                            numComponents,
+                                            whatType);
     
     FileInformation fileInfo(getFileName());
     const AString volumeFileName = FileInformation::assembleFileComponents(fileInfo.getAbsolutePath(),
@@ -1577,13 +1591,28 @@ ImageFile::convertToVolumeFile(const CONVERT_TO_VOLUME_COLOR_MODE colorMode,
     }
     
     volumeFile->clearVoxelColoringForMap(mapIndex);
-    volumeFile->updateScalarColoringForMap(mapIndex,
-                                           paletteFile);
-
+    volumeFile->updateScalarColoringForMap(mapIndex);
+    
     return volumeFile;
 }
 
+/**
+ * @return The control point file.
+ */
+ControlPointFile*
+ImageFile::getControlPointFile()
+{
+    return m_controlPointFile;
+}
 
+/**
+ * @return The control point file.
+ */
+const ControlPointFile*
+ImageFile::getControlPointFile() const
+{
+    return m_controlPointFile;
+}
 
 /**
  * Save file data from the scene.  For subclasses that need to
@@ -1599,11 +1628,13 @@ ImageFile::convertToVolumeFile(const CONVERT_TO_VOLUME_COLOR_MODE colorMode,
  *     sceneClass to which data members should be added.
  */
 void
-ImageFile::saveFileDataToScene(const SceneAttributes* /*sceneAttributes*/,
+ImageFile::saveFileDataToScene(const SceneAttributes* sceneAttributes,
                                    SceneClass* sceneClass)
 {
-    sceneClass->addFloat("m_windowZ",
-                         m_windowZ);
+    if (m_controlPointFile != NULL) {
+        sceneClass->addClass(m_controlPointFile->saveToScene(sceneAttributes,
+                                                             "m_controlPointFile"));
+    }
 }
 
 /**
@@ -1621,11 +1652,11 @@ ImageFile::saveFileDataToScene(const SceneAttributes* /*sceneAttributes*/,
  *     this interface.  Will NEVER be NULL.
  */
 void
-ImageFile::restoreFileDataFromScene(const SceneAttributes* /*sceneAttributes*/,
+ImageFile::restoreFileDataFromScene(const SceneAttributes* sceneAttributes,
                                         const SceneClass* sceneClass)
 {
-    m_windowZ = sceneClass->getFloatValue("m_windowZ",
-                                          s_defaultWindowDepthPercentage);
+    m_controlPointFile->restoreFromScene(sceneAttributes,
+                                         sceneClass->getClass("m_controlPointFile"));
 }
 
 
