@@ -180,7 +180,7 @@ m_parentToolBar(parentToolBar)
     WuQtUtilities::setToolTipAndStatusTip(m_volumeSliceProjectionTypeEnumComboBox->getWidget(),
                                           "Chooses viewing orientation (oblique or orthogonal)");
     
-    m_obliqueMaskingAction = new QAction("M");
+    m_obliqueMaskingAction = new QAction("M", this);
     m_obliqueMaskingAction->setToolTip(VolumeSliceInterpolationEdgeEffectsMaskingEnum::getToolTip());
     m_obliqueMaskingAction->setCheckable(true);
     QObject::connect(m_obliqueMaskingAction, &QAction::triggered,
@@ -273,11 +273,8 @@ BrainBrowserWindowToolBarSliceSelection::updateContent(BrowserTabContent* browse
          * Test selected file to see if it is an oblique volume file (not a CIFTI file)
          */
         bool obliqueVolumeFlag = false;
-        VolumeFile* volumeFile = dynamic_cast<VolumeFile*>(vf);
-        if (volumeFile != NULL) {
-            if ( ! volumeFile->isPlumb()) {
-                obliqueVolumeFlag = true;
-            }
+        if ( ! vf->getVolumeSpace().isPlumb()) {
+            obliqueVolumeFlag = true;
         }
         
         /*
