@@ -95,6 +95,7 @@ OverlayViewController::OverlayViewController(const Qt::Orientation orientation,
         minComboBoxWidth = 50;
         maxComboBoxWidth = 100000;
     }
+    const QComboBox::SizeAdjustPolicy comboSizePolicy = QComboBox::AdjustToContentsOnFirstShow; //QComboBox::AdjustToContents;
 
     /*
      * Enabled Check Box
@@ -114,6 +115,7 @@ OverlayViewController::OverlayViewController(const Qt::Orientation orientation,
     QObject::connect(this->fileComboBox, SIGNAL(activated(int)),
                      this, SLOT(fileComboBoxSelected(int)));
     this->fileComboBox->setToolTip("Selects file for this overlay");
+    this->fileComboBox->setSizeAdjustPolicy(comboSizePolicy);
     
     /*
      * Map Index Spin Box
@@ -132,6 +134,7 @@ OverlayViewController::OverlayViewController(const Qt::Orientation orientation,
     QObject::connect(this->mapNameComboBox, SIGNAL(activated(int)),
                      this, SLOT(mapNameComboBoxSelected(int)));
     this->mapNameComboBox->setToolTip("Select map by its name");
+    this->mapNameComboBox->setSizeAdjustPolicy(comboSizePolicy);
     
     /*
      * Opacity double spin box
@@ -363,6 +366,16 @@ OverlayViewController::fileComboBoxSelected(int indx)
 
     updateOverlaySettingsEditor();
     updateViewController(this->overlay);
+    
+    if (file != NULL) {
+        if (file->isVolumeMappable()) {
+            /*
+             * Need to update slice indices/coords in toolbar.
+             */
+            EventManager::get()->sendEvent(EventUserInterfaceUpdate().setWindowIndex(browserWindowIndex).addToolBar().getPointer());
+        }
+    }
+    
     updateGraphicsWindow();
 }
 
@@ -397,6 +410,7 @@ OverlayViewController::mapIndexSpinBoxValueChanged(int indx)
     if (mapYoking != MapYokingGroupEnum::MAP_YOKING_GROUP_OFF) {
         EventMapYokingSelectMap selectMapEvent(mapYoking,
                                                file,
+                                               NULL,
                                                overlayIndex,
                                                overlay->isEnabled());
         EventManager::get()->sendEvent(selectMapEvent.getPointer());
@@ -443,6 +457,7 @@ OverlayViewController::mapNameComboBoxSelected(int indx)
     if (mapYoking != MapYokingGroupEnum::MAP_YOKING_GROUP_OFF) {
         EventMapYokingSelectMap selectMapEvent(mapYoking,
                                                file,
+                                               NULL,
                                                indx,
                                                overlay->isEnabled());
         EventManager::get()->sendEvent(selectMapEvent.getPointer());
@@ -484,6 +499,7 @@ OverlayViewController::enabledCheckBoxClicked(bool checked)
         
         EventMapYokingSelectMap selectMapEvent(mapYoking,
                                                myFile,
+                                               NULL,
                                                myIndex,
                                                overlay->isEnabled());
         EventManager::get()->sendEvent(selectMapEvent.getPointer());
@@ -612,6 +628,7 @@ OverlayViewController::updateViewController(Overlay* overlay)
     FilePathNamePrefixCompactor::removeMatchingPathPrefixFromCaretDataFiles(dataFiles,
                                                                             displayNames);
     CaretAssert(dataFiles.size() == displayNames.size());
+
     /*
      * Load the file selection combo box.
      */
@@ -682,13 +699,16 @@ OverlayViewController::updateViewController(Overlay* overlay)
     bool haveMultipleMaps = false;
     bool dataIsMappedWithPalette = false;
     bool dataIsMappedWithLabelTable = false;
+    bool dataIsMappedWithRGBA = false;
     bool haveOpacity = false;
     if (haveFile) {
         dataIsMappedWithPalette = selectedFile->isMappedWithPalette();
         dataIsMappedWithLabelTable = selectedFile->isMappedWithLabelTable();
+        dataIsMappedWithRGBA    = selectedFile->isMappedWithRGBA();
         haveMultipleMaps = (selectedFile->getNumberOfMaps() > 1);
         haveOpacity = (dataIsMappedWithLabelTable
-                       || dataIsMappedWithPalette);
+                       || dataIsMappedWithPalette
+                       || dataIsMappedWithRGBA);
     }
     
     /**
@@ -825,9 +845,9 @@ OverlayViewController::createConstructionMenu(QWidget* parent)
     
     menu->addSeparator();
     
-    menu->addAction("Copy Path and File Name to Clipboard",
-                    this,
-                    SLOT(menuCopyFileNameToClipBoard()));
+    m_copyPathAndFileNameToClipboardAction = menu->addAction("Copy Path and File Name to Clipboard",
+                                                             this,
+                                                             SLOT(menuCopyFileNameToClipBoard()));
     
     menu->addAction("Copy Map Name to Clipboard",
                     this,
@@ -848,7 +868,7 @@ OverlayViewController::menuConstructionAboutToShow()
         int32_t mapIndex = -1;
         this->overlay->getSelectionData(caretDataFile,
                                         mapIndex);
-        
+    
         QString menuText = "Reload Selected File";
         if (caretDataFile != NULL) {
             if (caretDataFile->isModified()) {
@@ -860,7 +880,12 @@ OverlayViewController::menuConstructionAboutToShow()
                 }
                 menuText += suffix;
             }
+            
+            const bool notDynConnFileFlag = (caretDataFile->getDataFileType() != DataFileTypeEnum::CONNECTIVITY_DENSE_DYNAMIC);
+            m_constructionReloadFileAction->setEnabled(notDynConnFileFlag);
+            m_copyPathAndFileNameToClipboardAction->setEnabled(notDynConnFileFlag);
         }
+        
         m_constructionReloadFileAction->setText(menuText);
     }
 }
