@@ -42,10 +42,14 @@ namespace caret {
         
         virtual CursorEnum::Enum getCursor() const;
         
+        virtual void mouseLeftDoubleClick(const MouseEvent& mouseEvent);
+        
         virtual void mouseLeftClick(const MouseEvent& mouseEvent);
         
         virtual void mouseLeftClickWithShift(const MouseEvent& mouseEvent);
         
+        virtual void mouseLeftClickWithCtrlShift(const MouseEvent& mouseEvent);
+        
         virtual void mouseLeftDrag(const MouseEvent& mouseEvent);
         
         virtual void mouseLeftDragWithAlt(const MouseEvent& mouseEvent);
@@ -54,6 +58,10 @@ namespace caret {
         
         virtual void mouseLeftDragWithShift(const MouseEvent& mouseEvent);
 
+        virtual void showContextMenu(const MouseEvent& mouseEvent,
+                                     const QPoint& menuPosition,
+                                     BrainOpenGLWidget* openGLWidget);
+        
     protected:
         UserInputModeView(const UserInputMode inputMode);
         
