@@ -24,6 +24,7 @@
 #include "ByteSwapping.h"
 #include "CaretAssert.h"
 #include "CaretBinaryFile.h"
+#include "CaretMutex.h"
 #include "DataFileException.h"
 #include "NiftiHeader.h"
 
@@ -42,11 +43,14 @@ namespace caret
         NiftiHeader m_header;
         std::vector<int64_t> m_dims;
         std::vector<char> m_scratch;//scratch memory for byteswapping, type conversion, etc
+        CaretMutex m_mutex;//protect multithreaded calls from each other
         int numBytesPerElem();//for resizing scratch
         template<typename TO, typename FROM>
         void convertRead(TO* out, FROM* in, const int64_t& count);//for reading from file
         template<typename TO, typename FROM>
         void convertWrite(TO* out, const FROM* in, const int64_t& count);//for writing to file
+        template<typename TO, typename FROM>
+        static TO clamp(const FROM& in);//deal with integer cast being undefined when converting from outside range
     public:
         void openRead(const QString& filename);
         void writeNew(const QString& filename, const NiftiHeader& header, const int& version = 1, const bool& withRead = false, const bool& swapEndian = false);
@@ -82,13 +86,16 @@ namespace caret
             numSkip += indexSelect[curDim - fullDims] * numDimSkip;
             numDimSkip *= m_dims[curDim];
         }
+        CaretMutexLocker locked(&m_mutex);//protect starting with resizing until we are done converting, because we use an internal variable for scratch space
+        //we can't guarantee that the output memory is enough to use as scratch space, as we might be doing a narrowing conversion
+        //we are doing FILE ACCESS, so cpu performance isn't really something to worry about
         m_scratch.resize(numElems * numBytesPerElem());
         m_file.seek(numSkip * numBytesPerElem() + m_header.getDataOffset());
         int64_t numRead = 0;
         m_file.read(m_scratch.data(), m_scratch.size(), &numRead);
         if ((numRead != (int64_t)m_scratch.size() && !tolerateShortRead) || numRead < 0)//for now, assume read giving -1 is always a problem
         {
-            throw DataFileException("error while reading from file '" + m_file.getFilename() + "'");
+            throw DataFileException("error while reading from nifti file '" + m_file.getFilename() + "'");
         }
         switch (m_header.getDataType())
         {
@@ -153,6 +160,8 @@ namespace caret
             numSkip += indexSelect[curDim - fullDims] * numDimSkip;
             numDimSkip *= m_dims[curDim];
         }
+        CaretMutexLocker locked(&m_mutex);//protect starting with resizing until we are done writing, because we use an internal variable for scratch space
+        //we are doing FILE ACCESS, so cpu performance isn't really something to worry about
         m_scratch.resize(numElems * numBytesPerElem());
         m_file.seek(numSkip * numBytesPerElem() + m_header.getDataOffset());
         switch (m_header.getDataType())
@@ -216,12 +225,12 @@ namespace caret
             {
                 for (int64_t i = 0; i < count; ++i)
                 {
-                    out[i] = (TO)floor(0.5 + offset + mult * (long double)in[i]);//we don't always need that much precision, but it will still be faster than hard drives
+                    out[i] = clamp<TO, long double>(floor(0.5l + offset + mult * (long double)in[i]));//we don't always need that much precision, but it will still be faster than hard drives
                 }
             } else {
                 for (int64_t i = 0; i < count; ++i)
                 {
-                    out[i] = (TO)floor(0.5 + in[i]);
+                    out[i] = clamp<TO, double>(floor(0.5 + in[i]));
                 }
             }
         } else {
@@ -246,17 +255,17 @@ namespace caret
         double mult, offset;
         bool doScale = m_header.getDataScaling(mult, offset);
         if (std::numeric_limits<TO>::is_integer)//do round to nearest when integer output type
-        {
+        {//TODO: what about NaN?
             if (doScale)
             {
                 for (int64_t i = 0; i < count; ++i)
                 {
-                    out[i] = (TO)floor(0.5 + ((long double)in[i] - offset) / mult);//we don't always need that much precision, but it will still be faster than hard drives
+                    out[i] = clamp<TO, long double>(floor(0.5l + ((long double)in[i] - offset) / mult));//we don't always need that much precision, but it will still be faster than hard drives
                 }
             } else {
                 for (int64_t i = 0; i < count; ++i)
                 {
-                    out[i] = (TO)floor(0.5 + in[i]);
+                    out[i] = clamp<TO, double>(floor(0.5 + in[i]));
                 }
             }
         } else {
@@ -276,6 +285,20 @@ namespace caret
         if (m_header.isSwapped()) ByteSwapping::swapArray(out, count);
     }
     
+    template<typename TO, typename FROM>
+    TO NiftiIO::clamp(const FROM& in)
+    {
+        typedef std::numeric_limits<TO> mylimits;
+        if (mylimits::max() < in) return mylimits::max();
+        if (mylimits::lowest() > in) return mylimits::lowest();
+        /*if (mylimits::is_integer)//here is a c++03 solution to missing ::lowest
+        {
+            if (mylimits::min() > in) return mylimits::min();
+        } else {
+            if (-mylimits::max() > in) return -mylimits::max();
+        }//*/
+        return (TO)in;
+    }
 }
 
 #endif //__NIFTI_IO_H__
