@@ -37,13 +37,34 @@ using namespace caret;
 
 /**
  * Constructor.
+ *
+ * @param sceneType
+ *     Type of scene.
+ * @param scene
+ *     The scene.
  */
-SceneAttributes::SceneAttributes(const SceneTypeEnum::Enum sceneType)
-: CaretObject(), m_sceneType(sceneType)
+SceneAttributes::SceneAttributes(const SceneTypeEnum::Enum sceneType,
+                                 const Scene* scene)
+: CaretObject(), m_sceneType(sceneType), m_scene(scene)
 {
     m_restoreWindowBehaviorInSceneDisplay = RESTORE_WINDOW_POSITION_RELATIVE_TO_FIRST_AND_USE_SIZES;
     m_specFileNameSavedToScene   = true;
     m_allLoadedFilesSavedToScene = true;
+    m_useSceneForgroundAndBackgroundColorsFlag = true;
+    m_modifiedPaletteSettingsSavedToScene = true;//TSC: was uninitialized, bad idea
+}
+
+SceneAttributes::SceneAttributes(const SceneAttributes& rhs): CaretObject(), m_sceneType(rhs.m_sceneType)
+{
+    m_restoreWindowBehaviorInSceneDisplay = rhs.m_restoreWindowBehaviorInSceneDisplay;
+    m_specFileNameSavedToScene = rhs.m_specFileNameSavedToScene;
+    m_allLoadedFilesSavedToScene = rhs.m_allLoadedFilesSavedToScene;
+    m_useSceneForgroundAndBackgroundColorsFlag = rhs.m_useSceneForgroundAndBackgroundColorsFlag;
+    m_sceneName = rhs.m_sceneName;
+    //leaving sceneFileName empty, as that shouldn't even be stored inside the scene, and we don't know where this scene will be put
+    m_indicesOfTabsForSavingToScene = rhs.m_indicesOfTabsForSavingToScene;
+    m_indicesOfWindowsForSavingToScene = rhs.m_indicesOfWindowsForSavingToScene;
+    //leaving error message empty, seems to make the most sense
 }
 
 /**
@@ -63,15 +84,27 @@ SceneAttributes::getSceneType() const
     return m_sceneType;
 }
 
+/**
+ * @return The scene using these attributes.
+ */
+const Scene*
+SceneAttributes::getScene() const
+{
+    return m_scene;
+}
+
+
 /**
  * Set the indices of the tabs that are to be saved to the scene.
  * @param tabIndices
  *    Indices of tabs that are saved to the scene.
  */
 void 
-SceneAttributes::setIndicesOfTabsForSavingToScene(const std::vector<int32_t>& tabIndices)
+SceneAttributes::setIndicesOfTabsAndWindowsForSavingToScene(const std::vector<int32_t>& tabIndices,
+                                                  const std::vector<int32_t>& windowIndices)
 {
-    m_indicesOfTabsForSavingToScene = tabIndices;
+    m_indicesOfTabsForSavingToScene    = tabIndices;
+    m_indicesOfWindowsForSavingToScene = windowIndices;
 }
 
 /**
@@ -83,6 +116,15 @@ SceneAttributes::getIndicesOfTabsForSavingToScene() const
     return m_indicesOfTabsForSavingToScene;
 }
 
+/**
+ * @return Indices of tabs for saving to the scene.
+ */
+std::vector<int32_t>
+SceneAttributes::getIndicesOfWindowsForSavingToScene() const
+{
+    return m_indicesOfWindowsForSavingToScene;
+}
+
 /**
  * Is the given tab index for saving to the scene?
  *
@@ -103,6 +145,26 @@ SceneAttributes::isTabIndexSavedToScene(const int32_t tabIndex) const
     return false;
 }
 
+/**
+ * Is the given window index for saving to the scene?
+ *
+ * @param windowIndex
+ *    The window index.
+ * @return
+ *    True if saved to scene, else false.
+ */
+bool
+SceneAttributes::isWindowIndexSavedToScene(const int32_t windowIndex) const
+{
+    if (std::find(m_indicesOfWindowsForSavingToScene.begin(),
+                  m_indicesOfWindowsForSavingToScene.end(),
+                  windowIndex) != m_indicesOfWindowsForSavingToScene.end()) {
+        return true;
+    }
+    
+    return false;
+}
+
 /**
  * @return Name of the scene file.
  * May only have a value when restoring.
@@ -281,5 +343,31 @@ SceneAttributes::clearErrorMessage()
     m_errorMessage.clear();
 }
 
+/**
+ * @return Use the foreground and background colors from the scene?
+ *
+ * Note that scenes prior May 2016 may not contain foreground and
+ * background colors.
+ */
+bool
+SceneAttributes::isUseSceneForegroundAndBackgroundColors() const
+{
+    return m_useSceneForgroundAndBackgroundColorsFlag;
+}
+
+/**
+ * Set use the foreground and background colors from the scene.
+ *
+ * Note that scenes prior May 2016 may not contain foreground and
+ * background colors.
+ *
+ * @param status
+ *     New status for using colors from scene.
+ */
+void
+SceneAttributes::setUseSceneForegroundAndBackgroundColors(const bool status)
+{
+    m_useSceneForgroundAndBackgroundColorsFlag = status;
+}
 
 
