@@ -73,12 +73,16 @@ namespace caret {
             STATE_NONE,
             /// processing Scene Info tag
             STATE_SCENE_INFO,
+            /// processing Balsa Scene ID tag
+            STATE_SCENE_INFO_BALSA_ID,
             /// processing Scene Info Name tag
             STATE_SCENE_INFO_NAME,
             /// processing Scene Info Description tag
             STATE_SCENE_INFO_DESCRIPTION,
             /// processing Scene Info thumbnail tag
-            STATE_SCENE_INFO_IMAGE_THUMBNAIL
+            STATE_SCENE_INFO_IMAGE_THUMBNAIL,
+            /// process an unrecognized element in Scene Info
+            STATE_SCENE_INFO_UNRECOGNIZED
         };
         
         /// name of scene file
